/* $Id: mlog.h 548 2008-05-14 10:14:13Z gloomy $ */

#ifndef __PARSEC_LOG_H_INCLUDED__
#define __PARSEC_LOG_H_INCLUDED__

#include <sys/types.h>
#include <stdint.h>

#include <parsec/common.h>

#ifndef PROJECT
#error PROJECT is not defined
#endif
#define MLOG_CONFIG_DIR "/etc/" PROJECT "/mlog"
#define MLOG_PLUGINS_DIR "/usr/lib/mlog"

#define MLOG_PLUGIN_INIT int mlog_plugin_init(void)
#define MLOG_PLUGIN_DONE void mlog_plugin_done(void)

typedef uint32_t eid_t; /* audit event id */

#define PARSEC_LOG_EID_INVALID	(0)

#define PARSEC_LOG_EID_SOURCE(v)	(((v)&0xf0000000)>>28)
#define PARSEC_LOG_EID_NVAR(v)	((v)&0xf)
#define PARSEC_LOG_EID_NVAR_MAX	(0xf)

#define PARSEC_LOG_EID_SOURCE_USER		0
#define PARSEC_LOG_EID_SOURCE_KERNEL_PS		1
#define PARSEC_LOG_EID_SOURCE_KERNEL_FS		2
#define PARSEC_LOG_EID_SOURCE_USER_CUSTOM		3
/* 4-0xf -- reserved */
#define PARSEC_LOG_EID_SOURCE_UNKNOWN		4
#define PARSEC_LOG_EID_SOURCE_MAX			0xf
#define PARSEC_LOG_EID_SOURCE_NUM			(PARSEC_LOG_EID_SOURCE_MAX + 1)

#define PARSEC_LOG_EID(id,s,n)		\
	((((PARSEC_LOG_EID_SOURCE_##s)&0xf)<<28) |\
	 ((n) &0xf) | (id<<4))

#define _S(s)	strlen((s)), (s)
#define	_A(a)	sizeof(*(a)), (a)
#define _V(v)	sizeof(__typeof__(v)), &(v) 
#define _D(a,s) (s), (a)

#define PARSEC_LOG_STATUS_SUCCESS		0
#define PARSEC_LOG_STATUS_FAIL		1

#define PARSEC_LOG_SEND(name, status, arg...) \
	parsec_log_send(PARSEC_LOG_EVENT_##name,\
			PARSEC_LOG_STATUS_##status,##arg)
/* example:
   PARSEC_AUDIT_SEND(OPEN, SUCCESS, _V("/etc/shadow"),_V(mode));
extern int parsec_log_send(eid_t, uint16_t status, ...); 
   parameters as pairs: size vs data and so on... */

#define PARSEC_LOG_MAGIC		0x10121980
#define PARSEC_LOG_MAGIC_AE	0x80191210

/* Parser interface */

#define PARSEC_LOG_ETID_CLASS(v)	(((v)&0xf0000000)>>28)
#define PARSEC_LOG_ETID_CLASS_GENERIC	0
#define PARSEC_LOG_ETID_CLASS_CUSTOM	1
#define PARSEC_LOG_ETID(i,c) ((PARSEC_LOG_ETID_CLASS_##c<<28)|(i))
#define PARSEC_LOG_ETID_INVALID		(0)

#define PARSEC_LOG_KERN_SRC "/parsecfs/log-kernel" // source of kernel audit
#define PARSEC_LOG_USER_SRC "/parsecfs/log-user" // source & destination of user audit

typedef uint32_t 	etid_t; /* event data type id */

struct __parsec_log;
typedef struct __parsec_log *parsec_log_t;

typedef struct {
	int		type;
	size_t		size;
	void		*data;
} mlog_data_t;

/* type is: */
#define PARSEC_LOG_PARAMETER	0
#define PARSEC_LOG_CALLER		1 /* caller string */
#define PARSEC_LOG_EXTENDED	7 /* for extensions */

typedef struct	{
	char	*name;
	etid_t	etid;
	/* returns <0 if error, or size_t to fit data if not enough size
	   or res == NULL */
	char	*(*data2txt)(const mlog_data_t *data);
	int	(*alien2cpu)(mlog_data_t *ed);
} mlog_type_handler_t;

typedef struct {
	char 	*name;
	eid_t	eid;
	etid_t 	tids[PARSEC_LOG_EID_NVAR_MAX];
} mlog_event_handler_t;

#ifdef __cplusplus
extern "C" {
#endif
	extern int	mlog_init(void);
	extern void	mlog_done(void);
	extern int	mlog_register_type(mlog_type_handler_t *h);
	extern int	mlog_unregister_type(mlog_type_handler_t *h);
	extern int	mlog_register_event(mlog_event_handler_t *e);
	extern int	mlog_unregister_event(mlog_event_handler_t *e);
	extern size_t 	mlog_record_lookup(void *data, size_t size);
	extern int	mlog_compile(parsec_log_t *plog, 
				void *ev, size_t size);
	static inline int mlog_record_minsize(void) { return mlog_compile(NULL, NULL, 0); }
	static inline void mlog_free(parsec_log_t plog) { free (plog); }
	extern void	mlog_ae2cpu(void *data, size_t size);
	extern char	*mlog_txt_malloc(parsec_log_t);
	extern ssize_t  mlog_txt_fmt(parsec_log_t plog, char *ptr, ssize_t size, const char *fmt);
	extern int	parsec_log_send(eid_t eid, uint16_t status, ...);
	void		*mlog_cfg_register_events(const char *fname);
	void		mlog_cfg_unregister_events(void *han);
	void		*mlog_plugin_load(const char *fname);
	void		mlog_plugin_unload(void *);
/* lookup functions */
	extern pid_t 	mlog_pid (parsec_log_t plog);
	extern pid_t 	mlog_ppid (parsec_log_t plog);
	extern uint32_t	mlog_time (parsec_log_t);
	extern eid_t	mlog_event (parsec_log_t);
	extern int 	mlog_status (parsec_log_t);
	extern uid_t	mlog_uid (parsec_log_t);
	extern gid_t	mlog_gid (parsec_log_t);
	extern uid_t	mlog_euid (parsec_log_t);
	extern char	*mlog_arg_malloc (parsec_log_t, int num);
	extern char	*mlog_name_malloc (parsec_log_t);
	extern char	*mlog_caller_malloc (parsec_log_t);
	extern eid_t    mlog_eid_by_name(int source, const char *name);
	extern etid_t   mlog_etid_by_name(const char *name);
	ssize_t		mlog_listevents(int source, char **list);
	ssize_t		mlog_listtypes(char **list);
#ifdef __cplusplus
}
#endif

/* user mode events */
#define PARSEC_LOG_EVENT_MESSAGE PARSEC_LOG_EID(1,USER,1)

#endif // __PARSEC_LOG_H_INCLUDED__

